var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as events from 'events';
import * as protocol from 'vscode-languageserver-protocol';
import { ConsoleLogger, listen } from 'vscode-ws-jsonrpc';
import { registerServerCapability, unregisterServerCapability } from './server-capability-registration';
/**
 * Changes as compared to upstream:
 *  - markdown is preferred over plaintext
 *  - informative members are public and others are protected, not private
 *  - onServerInitialized() was extracted; it also emits a message once connected
 *  - initializeParams() was extracted, and can be modified by subclasses
 *  - typescript 3.7 was adopted to clean up deep references
 */
var LspWsConnection = /** @class */ (function (_super) {
    __extends(LspWsConnection, _super);
    function LspWsConnection(options) {
        var _this = _super.call(this) || this;
        _this.isConnected = false;
        _this.isInitialized = false;
        _this.documentVersion = 0;
        _this.documentInfo = options;
        return _this;
    }
    /**
     * Initialize a connection over a web socket that speaks the LSP protocol
     */
    LspWsConnection.prototype.connect = function (socket) {
        var _this = this;
        this.socket = socket;
        listen({
            webSocket: this.socket,
            logger: new ConsoleLogger(),
            onConnection: function (connection) {
                connection.listen();
                _this.isConnected = true;
                _this.connection = connection;
                _this.sendInitialize();
                _this.connection.onNotification('textDocument/publishDiagnostics', function (params) {
                    _this.emit('diagnostic', params);
                });
                _this.connection.onNotification('window/showMessage', function (params) {
                    _this.emit('logging', params);
                });
                _this.connection.onRequest('client/registerCapability', function (params) {
                    params.registrations.forEach(function (capabilityRegistration) {
                        _this.serverCapabilities = registerServerCapability(_this.serverCapabilities, capabilityRegistration);
                    });
                    _this.emit('logging', params);
                });
                _this.connection.onRequest('client/unregisterCapability', function (params) {
                    params.unregisterations.forEach(function (capabilityUnregistration) {
                        _this.serverCapabilities = unregisterServerCapability(_this.serverCapabilities, capabilityUnregistration);
                    });
                    _this.emit('logging', params);
                });
                _this.connection.onRequest('window/showMessageRequest', function (params) {
                    _this.emit('logging', params);
                });
                _this.connection.onError(function (e) {
                    _this.emit('error', e);
                });
                _this.connection.onClose(function () {
                    _this.isConnected = false;
                });
            }
        });
        return this;
    };
    LspWsConnection.prototype.close = function () {
        if (this.connection) {
            this.connection.dispose();
        }
        this.socket.close();
    };
    LspWsConnection.prototype.getDocumentUri = function () {
        return this.documentInfo.documentUri;
    };
    /**
     * Initialization parameters to be sent to the language server.
     * Subclasses can overload this when adding more features.
     */
    LspWsConnection.prototype.initializeParams = function () {
        return {
            capabilities: {
                textDocument: {
                    hover: {
                        dynamicRegistration: true,
                        contentFormat: ['markdown', 'plaintext']
                    },
                    synchronization: {
                        dynamicRegistration: true,
                        willSave: false,
                        didSave: false,
                        willSaveWaitUntil: false
                    },
                    completion: {
                        dynamicRegistration: true,
                        completionItem: {
                            snippetSupport: false,
                            commitCharactersSupport: true,
                            documentationFormat: ['markdown', 'plaintext'],
                            deprecatedSupport: false,
                            preselectSupport: false
                        },
                        contextSupport: false
                    },
                    signatureHelp: {
                        dynamicRegistration: true,
                        signatureInformation: {
                            documentationFormat: ['markdown', 'plaintext']
                        }
                    },
                    declaration: {
                        dynamicRegistration: true,
                        linkSupport: true
                    },
                    definition: {
                        dynamicRegistration: true,
                        linkSupport: true
                    },
                    typeDefinition: {
                        dynamicRegistration: true,
                        linkSupport: true
                    },
                    implementation: {
                        dynamicRegistration: true,
                        linkSupport: true
                    }
                },
                workspace: {
                    didChangeConfiguration: {
                        dynamicRegistration: true
                    }
                }
            },
            initializationOptions: null,
            processId: null,
            rootUri: this.documentInfo.rootUri,
            workspaceFolders: null
        };
    };
    LspWsConnection.prototype.sendInitialize = function () {
        var _this = this;
        if (!this.isConnected) {
            return;
        }
        var message = this.initializeParams();
        this.connection
            .sendRequest('initialize', message)
            .then(function (params) {
            _this.onServerInitialized(params);
        }, function (e) {
            console.warn('lsp-ws-connection initialization failure', e);
        });
    };
    LspWsConnection.prototype.sendChange = function () {
        if (!this.isConnected || !this.isInitialized) {
            return;
        }
        var textDocumentChange = {
            textDocument: {
                uri: this.documentInfo.documentUri,
                version: this.documentVersion
            },
            contentChanges: [
                {
                    text: this.documentInfo.documentText()
                }
            ]
        };
        this.connection.sendNotification('textDocument/didChange', textDocumentChange);
        this.documentVersion++;
    };
    LspWsConnection.prototype.getHoverTooltip = function (location) {
        var _this = this;
        if (!this.isInitialized) {
            return;
        }
        this.connection
            .sendRequest('textDocument/hover', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(function (params) {
            _this.emit('hover', params);
        });
    };
    LspWsConnection.prototype.getCompletion = function (location, token, triggerCharacter, triggerKind) {
        var _this = this;
        var _a;
        if (!this.isConnected || !((_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.completionProvider)) {
            return;
        }
        this.connection
            .sendRequest('textDocument/completion', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            },
            context: {
                triggerKind: triggerKind || protocol.CompletionTriggerKind.Invoked,
                triggerCharacter: triggerCharacter
            }
        })
            .then(function (params) {
            if (!params) {
                _this.emit('completion', params);
                return;
            }
            _this.emit('completion', 'items' in params ? params.items : params);
        });
    };
    LspWsConnection.prototype.getDetailedCompletion = function (completionItem) {
        var _this = this;
        if (!this.isConnected) {
            return;
        }
        this.connection
            .sendRequest('completionItem/resolve', completionItem)
            .then(function (result) {
            _this.emit('completionResolved', result);
        });
    };
    LspWsConnection.prototype.getSignatureHelp = function (location) {
        var _this = this;
        var _a, _b, _c;
        if (!this.isConnected || !((_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.signatureHelpProvider)) {
            return;
        }
        var code = this.documentInfo.documentText();
        var lines = code.split('\n');
        var typedCharacter = lines[location.line][location.ch];
        var triggers = ((_c = (_b = this.serverCapabilities) === null || _b === void 0 ? void 0 : _b.signatureHelpProvider) === null || _c === void 0 ? void 0 : _c.triggerCharacters) || [];
        if (triggers.indexOf(typedCharacter) === -1) {
            // Not a signature character
            return;
        }
        this.connection
            .sendRequest('textDocument/signatureHelp', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(function (params) {
            _this.emit('signature', params);
        });
    };
    /**
     * Request the locations of all matching document symbols
     */
    LspWsConnection.prototype.getDocumentHighlights = function (location) {
        var _this = this;
        var _a;
        if (!this.isConnected ||
            !((_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.documentHighlightProvider)) {
            return;
        }
        this.connection
            .sendRequest('textDocument/documentHighlight', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(function (params) {
            _this.emit('highlight', params);
        });
    };
    /**
     * Request a link to the definition of the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    LspWsConnection.prototype.getDefinition = function (location) {
        var _this = this;
        if (!this.isConnected || !this.isDefinitionSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/definition', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(function (result) {
            _this.emit('goTo', result);
        });
    };
    /**
     * Request a link to the type definition of the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    LspWsConnection.prototype.getTypeDefinition = function (location) {
        var _this = this;
        if (!this.isConnected || !this.isTypeDefinitionSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/typeDefinition', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(function (result) {
            _this.emit('goTo', result);
        });
    };
    /**
     * Request a link to the implementation of the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    LspWsConnection.prototype.getImplementation = function (location) {
        var _this = this;
        if (!this.isConnected || !this.isImplementationSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/implementation', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(function (result) {
            _this.emit('goTo', result);
        });
    };
    /**
     * Request a link to all references to the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    LspWsConnection.prototype.getReferences = function (location) {
        var _this = this;
        if (!this.isConnected || !this.isReferencesSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/references', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(function (result) {
            _this.emit('goTo', result);
        });
    };
    /**
     * The characters that trigger completion automatically.
     */
    LspWsConnection.prototype.getLanguageCompletionCharacters = function () {
        var _a, _b;
        return ((_b = (_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.completionProvider) === null || _b === void 0 ? void 0 : _b.triggerCharacters) || [];
    };
    /**
     * The characters that trigger signature help automatically.
     */
    LspWsConnection.prototype.getLanguageSignatureCharacters = function () {
        var _a, _b;
        return (((_b = (_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.signatureHelpProvider) === null || _b === void 0 ? void 0 : _b.triggerCharacters) || []);
    };
    /**
     * Does the server support go to definition?
     */
    LspWsConnection.prototype.isDefinitionSupported = function () {
        var _a;
        return !!((_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.definitionProvider);
    };
    /**
     * Does the server support go to type definition?
     */
    LspWsConnection.prototype.isTypeDefinitionSupported = function () {
        var _a;
        return !!((_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.typeDefinitionProvider);
    };
    /**
     * Does the server support go to implementation?
     */
    LspWsConnection.prototype.isImplementationSupported = function () {
        var _a;
        return !!((_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.implementationProvider);
    };
    /**
     * Does the server support find all references?
     */
    LspWsConnection.prototype.isReferencesSupported = function () {
        var _a;
        return !!((_a = this.serverCapabilities) === null || _a === void 0 ? void 0 : _a.referencesProvider);
    };
    LspWsConnection.prototype.onServerInitialized = function (params) {
        this.isInitialized = true;
        this.serverCapabilities = params.capabilities;
        var textDocumentMessage = {
            textDocument: {
                uri: this.documentInfo.documentUri,
                languageId: this.documentInfo.languageId,
                text: this.documentInfo.documentText(),
                version: this.documentVersion
            }
        };
        this.connection.sendNotification('initialized');
        this.connection.sendNotification('workspace/didChangeConfiguration', {
            settings: {}
        });
        this.connection.sendNotification('textDocument/didOpen', textDocumentMessage);
        this.sendChange();
        this.emit('serverInitialized', this.serverCapabilities);
    };
    return LspWsConnection;
}(events.EventEmitter));
export { LspWsConnection };
//# sourceMappingURL=ws-connection.js.map